@@ -7,3 +7,5 @@
 #undef HAVE_LIBHISTORY
 #undef HAVE_LIBREADLINE
 #undef SOCKLEN_T
+#undef HAVE_LIBPTHREAD
+#undef HAVE_PTHREAD_H
