@@ -7,6 +7,7 @@
  * Daniel Veillard <daniel@veillard.com>
  */
 
+#define IN_LIBXML
 #include "libxml.h"
 #ifdef LIBXML_DEBUG_ENABLED
 
@@ -27,6 +28,7 @@
 #include <libxml/HTMLparser.h>
 #include <libxml/xmlerror.h>
 #include <libxml/globals.h>
+#include <libxml/xpathInternals.h>
 
 /**
  * xmlDebugDumpString:
@@ -40,6 +42,8 @@ xmlDebugDumpString(FILE * output, const xmlChar * str)
 {
     int i;
 
+    if (output == NULL)
+	output = stdout;
     if (str == NULL) {
         fprintf(output, "(NULL)");
         return;
@@ -67,6 +71,11 @@ xmlDebugDumpDtdNode(FILE *output, xmlDtdPtr dtd, int depth) {
 
     fprintf(output, shift);
 
+    if (dtd == NULL) {
+	fprintf(output, "DTD node is NULL\n");
+	return;
+    }
+
     if (dtd->type != XML_DTD_NODE) {
 	fprintf(output, "PBM: not a DTD\n");
 	return;
@@ -84,24 +93,24 @@ xmlDebugDumpDtdNode(FILE *output, xmlDtdPtr dtd, int depth) {
      * Do a bit of checking
      */
     if (dtd->parent == NULL)
-	fprintf(output, "PBM: Dtd has no parent\n");
+	fprintf(output, "PBM: DTD has no parent\n");
     if (dtd->doc == NULL)
-	fprintf(output, "PBM: Dtd has no doc\n");
+	fprintf(output, "PBM: DTD has no doc\n");
     if ((dtd->parent != NULL) && (dtd->doc != dtd->parent->doc))
-	fprintf(output, "PBM: Dtd doc differs from parent's one\n");
+	fprintf(output, "PBM: DTD doc differs from parent's one\n");
     if (dtd->prev == NULL) {
 	if ((dtd->parent != NULL) && (dtd->parent->children != (xmlNodePtr)dtd))
-	    fprintf(output, "PBM: Dtd has no prev and not first of list\n");
+	    fprintf(output, "PBM: DTD has no prev and not first of list\n");
     } else {
 	if (dtd->prev->next != (xmlNodePtr) dtd)
-	    fprintf(output, "PBM: Dtd prev->next : back link wrong\n");
+	    fprintf(output, "PBM: DTD prev->next : back link wrong\n");
     }
     if (dtd->next == NULL) {
 	if ((dtd->parent != NULL) && (dtd->parent->last != (xmlNodePtr) dtd))
-	    fprintf(output, "PBM: Dtd has no next and not last of list\n");
+	    fprintf(output, "PBM: DTD has no next and not last of list\n");
     } else {
 	if (dtd->next->prev != (xmlNodePtr) dtd)
-	    fprintf(output, "PBM: Dtd next->prev : forward link wrong\n");
+	    fprintf(output, "PBM: DTD next->prev : forward link wrong\n");
     }
 }
 
@@ -116,6 +125,10 @@ xmlDebugDumpAttrDecl(FILE *output, xmlAttributePtr attr, int depth) {
 
     fprintf(output, shift);
 
+    if (attr == NULL) {
+	fprintf(output, "Attribute declaration is NULL\n");
+	return;
+    }
     if (attr->type != XML_ATTRIBUTE_DECL) {
 	fprintf(output, "PBM: not a Attr\n");
 	return;
@@ -233,6 +246,10 @@ xmlDebugDumpElemDecl(FILE *output, xmlElementPtr elem, int depth) {
 
     fprintf(output, shift);
 
+    if (elem == NULL) {
+	fprintf(output, "Element declaration is NULL\n");
+	return;
+    }
     if (elem->type != XML_ELEMENT_DECL) {
 	fprintf(output, "PBM: not a Elem\n");
 	return;
@@ -307,6 +324,10 @@ xmlDebugDumpEntityDecl(FILE *output, xmlEntityPtr ent, int depth) {
 
     fprintf(output, shift);
 
+    if (ent == NULL) {
+	fprintf(output, "Entity declaration is NULL\n");
+	return;
+    }
     if (ent->type != XML_ENTITY_DECL) {
 	fprintf(output, "PBM: not a Entity decl\n");
 	return;
@@ -391,6 +412,11 @@ xmlDebugDumpNamespace(FILE *output, xmlNsPtr ns, int depth) {
     shift[2 * i] = shift[2 * i + 1] = 0;
 
     fprintf(output, shift);
+
+    if (ns == NULL) {
+	fprintf(output, "namespace node is NULL\n");
+	return;
+    }
     if (ns->type != XML_NAMESPACE_DECL) {
         fprintf(output, "invalid namespace node %d\n", ns->type);
 	return;
@@ -429,6 +455,11 @@ xmlDebugDumpEntity(FILE *output, xmlEntityPtr ent, int depth) {
     shift[2 * i] = shift[2 * i + 1] = 0;
 
     fprintf(output, shift);
+    
+    if (ent == NULL) {
+	fprintf(output, "Entity is NULL\n");
+	return;
+    }
     switch (ent->etype) {
         case XML_INTERNAL_GENERAL_ENTITY:
 	    fprintf(output, "INTERNAL_GENERAL_ENTITY ");
@@ -487,7 +518,11 @@ xmlDebugDumpAttr(FILE *output, xmlAttrPtr attr, int depth) {
     shift[2 * i] = shift[2 * i + 1] = 0;
 
     fprintf(output, shift);
-
+    
+    if (attr == NULL) {
+	fprintf(output, "Attr is NULL");
+	return;
+    }
     fprintf(output, "ATTRIBUTE ");
     xmlDebugDumpString(output, attr->name);
     fprintf(output, "\n");
@@ -527,6 +562,8 @@ xmlDebugDumpAttr(FILE *output, xmlAttrPtr attr, int depth) {
 void
 xmlDebugDumpAttrList(FILE * output, xmlAttrPtr attr, int depth)
 {
+    if (output == NULL)
+	output = stdout;
     while (attr != NULL) {
         xmlDebugDumpAttr(output, attr, depth);
         attr = attr->next;
@@ -547,10 +584,17 @@ xmlDebugDumpOneNode(FILE * output, xmlNodePtr node, int depth)
     int i;
     char shift[100];
 
+    if (output == NULL)
+	output = stdout;
     for (i = 0; ((i < depth) && (i < 25)); i++)
         shift[2 * i] = shift[2 * i + 1] = ' ';
     shift[2 * i] = shift[2 * i + 1] = 0;
 
+    if (node == NULL) {
+	fprintf(output, shift);
+	fprintf(output, "node is NULL\n");
+	return;
+    }
     switch (node->type) {
         case XML_ELEMENT_NODE:
             fprintf(output, shift);
@@ -652,11 +696,7 @@ xmlDebugDumpOneNode(FILE * output, xmlNodePtr node, int depth)
             shift[2 * i + 2] = shift[2 * i + 3] = 0;
             fprintf(output, shift);
             fprintf(output, "content=");
-#ifndef XML_USE_BUFFER_CONTENT
             xmlDebugDumpString(output, node->content);
-#else
-            xmlDebugDumpString(output, xmlBufferContent(node->content));
-#endif
             fprintf(output, "\n");
         }
     } else {
@@ -704,6 +744,20 @@ xmlDebugDumpOneNode(FILE * output, xmlNodePtr node, int depth)
 void
 xmlDebugDumpNode(FILE * output, xmlNodePtr node, int depth)
 {
+    if (output == NULL)
+	output = stdout;
+    if (node == NULL) {
+	int i;
+	char shift[100];
+    	
+	for (i = 0; ((i < depth) && (i < 25)); i++)
+	    shift[2 * i] = shift[2 * i + 1] = ' ';
+	shift[2 * i] = shift[2 * i + 1] = 0;
+	
+	fprintf(output, shift);
+	fprintf(output, "node is NULL\n");
+	return;
+    }	
     xmlDebugDumpOneNode(output, node, depth);
     if ((node->children != NULL) && (node->type != XML_ENTITY_REF_NODE))
         xmlDebugDumpNodeList(output, node->children, depth + 1);
@@ -720,6 +774,8 @@ xmlDebugDumpNode(FILE * output, xmlNodePtr node, int depth)
 void
 xmlDebugDumpNodeList(FILE * output, xmlNodePtr node, int depth)
 {
+    if (output == NULL)
+	output = stdout;
     while (node != NULL) {
         xmlDebugDumpNode(output, node, depth);
         node = node->next;
@@ -845,8 +901,12 @@ xmlDebugDumpDocument(FILE * output, xmlDocPtr doc)
 void
 xmlDebugDumpDTD(FILE * output, xmlDtdPtr dtd)
 {
-    if (dtd == NULL)
+    if (output == NULL)
+	output = stdout;
+    if (dtd == NULL) {
+	fprintf(output, "DTD is NULL\n");
         return;
+    }
     if (dtd->type != XML_DTD_NODE) {
         fprintf(output, "PBM: not a DTD\n");
         return;
@@ -864,23 +924,23 @@ xmlDebugDumpDTD(FILE * output, xmlDtdPtr dtd)
      * Do a bit of checking
      */
     if ((dtd->parent != NULL) && (dtd->doc != dtd->parent->doc))
-        fprintf(output, "PBM: Dtd doc differs from parent's one\n");
+        fprintf(output, "PBM: DTD doc differs from parent's one\n");
     if (dtd->prev == NULL) {
         if ((dtd->parent != NULL)
             && (dtd->parent->children != (xmlNodePtr) dtd))
             fprintf(output,
-                    "PBM: Dtd has no prev and not first of list\n");
+                    "PBM: DTD has no prev and not first of list\n");
     } else {
         if (dtd->prev->next != (xmlNodePtr) dtd)
-            fprintf(output, "PBM: Dtd prev->next : back link wrong\n");
+            fprintf(output, "PBM: DTD prev->next : back link wrong\n");
     }
     if (dtd->next == NULL) {
         if ((dtd->parent != NULL)
             && (dtd->parent->last != (xmlNodePtr) dtd))
-            fprintf(output, "PBM: Dtd has no next and not last of list\n");
+            fprintf(output, "PBM: DTD has no next and not last of list\n");
     } else {
         if (dtd->next->prev != (xmlNodePtr) dtd)
-            fprintf(output, "PBM: Dtd next->prev : forward link wrong\n");
+            fprintf(output, "PBM: DTD next->prev : forward link wrong\n");
     }
     if (dtd->children == NULL)
         fprintf(output, "    DTD is empty\n");
@@ -890,6 +950,10 @@ xmlDebugDumpDTD(FILE * output, xmlDtdPtr dtd)
 
 static void
 xmlDebugDumpEntityCallback(xmlEntityPtr cur, FILE *output) {
+    if (cur == NULL) {
+	fprintf(output, "Entity is NULL");
+	return;
+    }
     fprintf(output, "%s : ", cur->name);
     switch (cur->etype) {
 	case XML_INTERNAL_GENERAL_ENTITY:
@@ -1003,9 +1067,21 @@ xmlDebugDumpEntities(FILE * output, xmlDocPtr doc)
         fprintf(output, "No entities in external subset\n");
 }
 
-int xmlLsCountNode(xmlNodePtr node) {
+/**
+ * xmlLsCountNode:
+ * @node:  the node to count
+ *
+ * Count the children of @node.
+ *
+ * Returns the number of children of @node.
+ */
+int
+xmlLsCountNode(xmlNodePtr node) {
     int ret = 0;
     xmlNodePtr list = NULL;
+    
+    if (node == NULL)
+	return(0);
 
     switch (node->type) {
 	case XML_ELEMENT_NODE:
@@ -1026,11 +1102,7 @@ int xmlLsCountNode(xmlNodePtr node) {
 	case XML_PI_NODE:
 	case XML_COMMENT_NODE:
 	    if (node->content != NULL) {
-#ifndef XML_USE_BUFFER_CONTENT	    
 		ret = xmlStrlen(node->content);
-#else
-		ret = xmlBufferLength(node->content);
-#endif
             }
 	    break;
 	case XML_ENTITY_REF_NODE:
@@ -1053,8 +1125,19 @@ int xmlLsCountNode(xmlNodePtr node) {
     return(ret);
 }
 
+/**
+ * xmlLsOneNode:
+ * @output:  the FILE * for the output
+ * @node:  the node to dump
+ *
+ * Dump to @output the type and name of @node.
+ */
 void
 xmlLsOneNode(FILE *output, xmlNodePtr node) {
+    if (node == NULL) {
+	fprintf(output, "NULL\n");
+	return;
+    }
     switch (node->type) {
 	case XML_ELEMENT_NODE:
 	    fprintf(output, "-");
@@ -1066,7 +1149,7 @@ xmlLsOneNode(FILE *output, xmlNodePtr node) {
 	    fprintf(output, "t");
 	    break;
 	case XML_CDATA_SECTION_NODE:
-	    fprintf(output, "c");
+	    fprintf(output, "C");
 	    break;
 	case XML_ENTITY_REF_NODE:
 	    fprintf(output, "e");
@@ -1095,17 +1178,22 @@ xmlLsOneNode(FILE *output, xmlNodePtr node) {
 	case XML_NOTATION_NODE:
 	    fprintf(output, "N");
 	    break;
+	case XML_NAMESPACE_DECL:
+	    fprintf(output, "n");
+	    break;
 	default:
 	    fprintf(output, "?");
     }
-    if (node->properties != NULL)
-	fprintf(output, "a");
-    else	
-	fprintf(output, "-");
-    if (node->nsDef != NULL) 
-	fprintf(output, "n");
-    else	
-	fprintf(output, "-");
+    if (node->type != XML_NAMESPACE_DECL) {
+	if (node->properties != NULL)
+	    fprintf(output, "a");
+	else	
+	    fprintf(output, "-");
+	if (node->nsDef != NULL) 
+	    fprintf(output, "n");
+	else	
+	    fprintf(output, "-");
+    }
 
     fprintf(output, " %8d ", xmlLsCountNode(node));
 
@@ -1120,11 +1208,7 @@ xmlLsOneNode(FILE *output, xmlNodePtr node) {
 	    break;
 	case XML_TEXT_NODE:
 	    if (node->content != NULL) {
-#ifndef XML_USE_BUFFER_CONTENT	    
 		xmlDebugDumpString(output, node->content);
-#else
-		xmlDebugDumpString(output, xmlBufferContent(node->content));
-#endif
             }
 	    break;
 	case XML_CDATA_SECTION_NODE:
@@ -1153,6 +1237,15 @@ xmlLsOneNode(FILE *output, xmlNodePtr node) {
 	    break;
 	case XML_NOTATION_NODE:
 	    break;
+	case XML_NAMESPACE_DECL: {
+	    xmlNsPtr ns = (xmlNsPtr) node;
+
+	    if (ns->prefix == NULL)
+		fprintf(output, "default -> %s", ns->href);
+	    else
+		fprintf(output, "%s -> %s", ns->prefix, ns->href);
+	    break;
+	}
 	default:
 	    if (node->name != NULL)
 		fprintf(output, "%s", node->name);
@@ -1162,10 +1255,12 @@ xmlLsOneNode(FILE *output, xmlNodePtr node) {
 
 /**
  * xmlBoolToText:
- * @boolval : a bool to turn into text
+ * @boolval: a bool to turn into text
  *
  * Convenient way to turn bool into text 
-*/
+ *
+ * Returns a pointer to either "True" or "False"
+ */
 const char *
 xmlBoolToText(int boolval)
 {
@@ -1191,9 +1286,9 @@ xmlBoolToText(int boolval)
  */
 
 /**
- * xmlShellPrintXpathError:
+ * xmlShellPrintXPathError:
  * @errorType: valid xpath error id
- * @arg : the argument that cause xpath to fail
+ * @arg: the argument that cause xpath to fail
  *
  * Print the xpath error to libxml default error channel
  */
@@ -1251,38 +1346,59 @@ xmlShellPrintXPathError(int errorType, const char *arg)
 
 
 /**
- * xmlShellPrintNode:
- * @node : a non-null node to print to stdout
+ * xmlShellPrintNodeCtxt:
+ * @ctxt : a non-null shell context
+ * @node : a non-null node to print to the output FILE
  *
- * Print node to stdout
+ * Print node to the output FILE
+ */
+static void
+xmlShellPrintNodeCtxt(xmlShellCtxtPtr ctxt,xmlNodePtr node)
+{
+    FILE *fp;
+
+    if (!node)
+        return;
+    if (ctxt == NULL)
+	fp = stdout;
+    else
+	fp = ctxt->output;
+
+    if (node->type == XML_DOCUMENT_NODE)
+        xmlDocDump(fp, (xmlDocPtr) node);
+    else if (node->type == XML_ATTRIBUTE_NODE)
+        xmlDebugDumpAttrList(fp, (xmlAttrPtr) node, 0);
+    else
+        xmlElemDump(fp, node->doc, node);
+
+    fprintf(fp, "\n");
+}
+
+/**
+ * xmlShellPrintNode:
+ * @node : a non-null node to print to the output FILE
+ *
+ * Print node to the output FILE
  */
 void
 xmlShellPrintNode(xmlNodePtr node)
 {
-    if (!node)
-        return;
-
-    if (node->type == XML_DOCUMENT_NODE)
-        xmlDocDump(stdout, (xmlDocPtr) node);
-    else if (node->type == XML_ATTRIBUTE_NODE)
-        xmlDebugDumpAttrList(stdout, (xmlAttrPtr) node, 0);
-    else
-        xmlElemDump(stdout, node->doc, node);
-
-    fprintf(stdout, "\n");
+    xmlShellPrintNodeCtxt(NULL, node);
 }
 
-
 /**
- * xmlShellPrintXPathResult:
- * list : a valid result generated by an xpath evaluation
+ * xmlShellPrintXPathResultCtxt:
+ * @ctxt: a valid shell context
+ * @list: a valid result generated by an xpath evaluation
  *
- * Prints result to stdout
+ * Prints result to the output FILE
  */
-void
-xmlShellPrintXPathResult(xmlXPathObjectPtr list)
+static void
+xmlShellPrintXPathResultCtxt(xmlShellCtxtPtr ctxt,xmlXPathObjectPtr list)
 {
     int i = 0;
+    if (!ctxt)
+       return;
 
     if (list != NULL) {
         switch (list->type) {
@@ -1294,8 +1410,8 @@ xmlShellPrintXPathResult(xmlXPathObjectPtr list)
                              indx++) {
                             if (i > 0)
                                 fprintf(stderr, " -------\n");
-                            xmlShellPrintNode(list->nodesetval->
-                                              nodeTab[indx]);
+                            xmlShellPrintNodeCtxt(ctxt,
+				    list->nodesetval->nodeTab[indx]);
                         }
                     } else {
                         xmlGenericError(xmlGenericErrorContext,
@@ -1323,6 +1439,18 @@ xmlShellPrintXPathResult(xmlXPathObjectPtr list)
     }
 }
 
+/**
+ * xmlShellPrintXPathResult:
+ * @list: a valid result generated by an xpath evaluation
+ *
+ * Prints result to the output FILE
+ */
+void
+xmlShellPrintXPathResult(xmlXPathObjectPtr list)
+{
+    xmlShellPrintXPathResultCtxt(NULL, list);
+}
+
 /**
  * xmlShellList:
  * @ctxt:  the shell context
@@ -1336,23 +1464,31 @@ xmlShellPrintXPathResult(xmlXPathObjectPtr list)
  * Returns 0
  */
 int
-xmlShellList(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
+xmlShellList(xmlShellCtxtPtr ctxt,
              char *arg ATTRIBUTE_UNUSED, xmlNodePtr node,
              xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
     xmlNodePtr cur;
-
+    if (!ctxt)
+        return (0);
+    if (node == NULL) {
+	fprintf(ctxt->output, "NULL\n");
+	return (0);
+    }
     if ((node->type == XML_DOCUMENT_NODE) ||
         (node->type == XML_HTML_DOCUMENT_NODE)) {
         cur = ((xmlDocPtr) node)->children;
+    } else if (node->type == XML_NAMESPACE_DECL) {
+        xmlLsOneNode(ctxt->output, node);
+        return (0);
     } else if (node->children != NULL) {
         cur = node->children;
     } else {
-        xmlLsOneNode(stdout, node);
+        xmlLsOneNode(ctxt->output, node);
         return (0);
     }
     while (cur != NULL) {
-        xmlLsOneNode(stdout, cur);
+        xmlLsOneNode(ctxt->output, cur);
         cur = cur->next;
     }
     return (0);
@@ -1371,23 +1507,50 @@ xmlShellList(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
  * Returns 0
  */
 int
-xmlShellBase(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
+xmlShellBase(xmlShellCtxtPtr ctxt,
              char *arg ATTRIBUTE_UNUSED, xmlNodePtr node,
              xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
     xmlChar *base;
+    if (!ctxt)
+        return 0;
+    if (node == NULL) {
+	fprintf(ctxt->output, "NULL\n");
+	return (0);
+    }    
 
     base = xmlNodeGetBase(node->doc, node);
 
     if (base == NULL) {
-        fprintf(stdout, " No base found !!!\n");
+        fprintf(ctxt->output, " No base found !!!\n");
     } else {
-        fprintf(stdout, "%s\n", base);
+        fprintf(ctxt->output, "%s\n", base);
         xmlFree(base);
     }
     return (0);
 }
 
+/**
+ * xmlShellSetBase:
+ * @ctxt:  the shell context
+ * @arg:  the new base
+ * @node:  a node
+ * @node2:  unused
+ *
+ * Implements the XML shell function "setbase"
+ * change the current XML base of the node
+ *
+ * Returns 0
+ */
+static int
+xmlShellSetBase(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
+             char *arg ATTRIBUTE_UNUSED, xmlNodePtr node,
+             xmlNodePtr node2 ATTRIBUTE_UNUSED)
+{
+    xmlNodeSetBase(node, (xmlChar*) arg);
+    return (0);
+}
+
 /**
  * xmlShellDir:
  * @ctxt:  the shell context
@@ -1405,13 +1568,19 @@ xmlShellDir(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
             char *arg ATTRIBUTE_UNUSED, xmlNodePtr node,
             xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
+    if (!ctxt)
+        return (0);
+    if (node == NULL) {
+	fprintf(ctxt->output, "NULL\n");
+	return (0);
+    }    
     if ((node->type == XML_DOCUMENT_NODE) ||
         (node->type == XML_HTML_DOCUMENT_NODE)) {
-        xmlDebugDumpDocumentHead(stdout, (xmlDocPtr) node);
+        xmlDebugDumpDocumentHead(ctxt->output, (xmlDocPtr) node);
     } else if (node->type == XML_ATTRIBUTE_NODE) {
-        xmlDebugDumpAttr(stdout, (xmlAttrPtr) node, 0);
+        xmlDebugDumpAttr(ctxt->output, (xmlAttrPtr) node, 0);
     } else {
-        xmlDebugDumpOneNode(stdout, node, 0);
+        xmlDebugDumpOneNode(ctxt->output, node, 0);
     }
     return (0);
 }
@@ -1432,25 +1601,31 @@ int
 xmlShellCat(xmlShellCtxtPtr ctxt, char *arg ATTRIBUTE_UNUSED,
             xmlNodePtr node, xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
+    if (!ctxt)
+        return (0);
+    if (node == NULL) {
+	fprintf(ctxt->output, "NULL\n");
+	return (0);
+    }    
     if (ctxt->doc->type == XML_HTML_DOCUMENT_NODE) {
 #ifdef LIBXML_HTML_ENABLED
         if (node->type == XML_HTML_DOCUMENT_NODE)
-            htmlDocDump(stdout, (htmlDocPtr) node);
+            htmlDocDump(ctxt->output, (htmlDocPtr) node);
         else
-            htmlNodeDumpFile(stdout, ctxt->doc, node);
+            htmlNodeDumpFile(ctxt->output, ctxt->doc, node);
 #else
         if (node->type == XML_DOCUMENT_NODE)
-            xmlDocDump(stdout, (xmlDocPtr) node);
+            xmlDocDump(ctxt->output, (xmlDocPtr) node);
         else
-            xmlElemDump(stdout, ctxt->doc, node);
+            xmlElemDump(ctxt->output, ctxt->doc, node);
 #endif /* LIBXML_HTML_ENABLED */
     } else {
         if (node->type == XML_DOCUMENT_NODE)
-            xmlDocDump(stdout, (xmlDocPtr) node);
+            xmlDocDump(ctxt->output, (xmlDocPtr) node);
         else
-            xmlElemDump(stdout, ctxt->doc, node);
+            xmlElemDump(ctxt->output, ctxt->doc, node);
     }
-    fprintf(stdout, "\n");
+    fprintf(ctxt->output, "\n");
     return (0);
 }
 
@@ -1481,7 +1656,7 @@ xmlShellLoad(xmlShellCtxtPtr ctxt, char *filename,
 #ifdef LIBXML_HTML_ENABLED
         doc = htmlParseFile(filename, NULL);
 #else
-        fprintf(stdout, "HTML support not compiled in\n");
+        fprintf(ctxt->output, "HTML support not compiled in\n");
         doc = NULL;
 #endif /* LIBXML_HTML_ENABLED */
     } else {
@@ -1515,7 +1690,7 @@ xmlShellLoad(xmlShellCtxtPtr ctxt, char *filename,
  * @node2:  unused
  *
  * Implements the XML shell function "write"
- * Write the current node to the filename, it saves the serailization
+ * Write the current node to the filename, it saves the serialization
  * of the subtree under the @node specified
  *
  * Returns 0 or -1 in case of error
@@ -1580,7 +1755,7 @@ xmlShellWrite(xmlShellCtxtPtr ctxt, char *filename, xmlNodePtr node,
 /**
  * xmlShellSave:
  * @ctxt:  the shell context
- * @filename:  the file name (optionnal)
+ * @filename:  the file name (optional)
  * @node:  unused
  * @node2:  unused
  *
@@ -1637,7 +1812,7 @@ xmlShellSave(xmlShellCtxtPtr ctxt, char *filename,
 /**
  * xmlShellValidate:
  * @ctxt:  the shell context
- * @dtd:  the DTD URI (optionnal)
+ * @dtd:  the DTD URI (optional)
  * @node:  unused
  * @node2:  unused
  *
@@ -1688,24 +1863,27 @@ xmlShellValidate(xmlShellCtxtPtr ctxt, char *dtd,
  * Returns 0 or -1 in case of error
  */
 int
-xmlShellDu(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
+xmlShellDu(xmlShellCtxtPtr ctxt,
            char *arg ATTRIBUTE_UNUSED, xmlNodePtr tree,
            xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
     xmlNodePtr node;
     int indent = 0, i;
 
+    if (!ctxt)
+	return (-1);
+
     if (tree == NULL)
         return (-1);
     node = tree;
     while (node != NULL) {
         if ((node->type == XML_DOCUMENT_NODE) ||
             (node->type == XML_HTML_DOCUMENT_NODE)) {
-            fprintf(stdout, "/\n");
+            fprintf(ctxt->output, "/\n");
         } else if (node->type == XML_ELEMENT_NODE) {
             for (i = 0; i < indent; i++)
-                fprintf(stdout, "  ");
-            fprintf(stdout, "%s\n", node->name);
+                fprintf(ctxt->output, "  ");
+            fprintf(ctxt->output, "%s\n", node->name);
         } else {
         }
 
@@ -1757,7 +1935,7 @@ xmlShellDu(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
  * xmlShellPwd:
  * @ctxt:  the shell context
  * @buffer:  the output buffer
- * @tree:  a node 
+ * @node:  a node 
  * @node2:  unused
  *
  * Implements the XML shell function "pwd"
@@ -1800,7 +1978,7 @@ xmlShellPwd(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED, char *buffer,
  * @doc:  the initial document
  * @filename:  the output buffer
  * @input:  the line reading function
- * @output:  the output FILE*
+ * @output:  the output FILE*, defaults to stdout if NULL
  *
  * Implements the XML shell 
  * This allow to load, validate, view, modify and save a document
@@ -1826,7 +2004,7 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
     if (input == NULL)
         return;
     if (output == NULL)
-        return;
+        output = stdout;
     ctxt = (xmlShellCtxtPtr) xmlMalloc(sizeof(xmlShellCtxt));
     if (ctxt == NULL)
         return;
@@ -1846,11 +2024,11 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 #endif /* LIBXML_XPATH_ENABLED */
     while (1) {
         if (ctxt->node == (xmlNodePtr) ctxt->doc)
-            sprintf(prompt, "%s > ", "/");
+            snprintf(prompt, sizeof(prompt), "%s > ", "/");
         else if (ctxt->node->name)
             snprintf(prompt, sizeof(prompt), "%s > ", ctxt->node->name);
         else
-            sprintf(prompt, "? > ");
+            snprintf(prompt, sizeof(prompt), "? > ");
         prompt[sizeof(prompt) - 1] = 0;
 
         /*
@@ -1904,22 +2082,26 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
         if (!strcmp(command, "bye"))
             break;
 		if (!strcmp(command, "help")) {
-		  fprintf(stdout, "\tbase         display XML base of the node\n");
-		  fprintf(stdout, "\tbye          leave shell\n");
-		  fprintf(stdout, "\tcat [node]   display node or current node\n");
-		  fprintf(stdout, "\tcd [path]    change directory to path or to root\n");
-		  fprintf(stdout, "\tdir [path]   dumps informations about the node (namespace, attributes, content)\n");
-		  fprintf(stdout, "\tdu [path]    show the structure of the subtree under path or the current node\n");
-		  fprintf(stdout, "\texit         leave shell\n");
-		  fprintf(stdout, "\thelp         display this help\n");
-		  fprintf(stdout, "\tfree         display memory usage\n");
-		  fprintf(stdout, "\tload [name]  load a new document with name\n");
-		  fprintf(stdout, "\tls [path]    list contents of path or the current directory\n");
-		  fprintf(stdout, "\tpwd          display current working directory\n");
-		  fprintf(stdout, "\tquit         leave shell\n");
-		  fprintf(stdout, "\tsave [name]  save this document to name or the original name\n");
-		  fprintf(stdout, "\tvalidate     check the document for errors\n");
-		  fprintf(stdout, "\twrite [name] write the current node to the filename\n");
+		  fprintf(ctxt->output, "\tbase         display XML base of the node\n");
+		  fprintf(ctxt->output, "\tsetbase URI  change the XML base of the node\n");
+		  fprintf(ctxt->output, "\tbye          leave shell\n");
+		  fprintf(ctxt->output, "\tcat [node]   display node or current node\n");
+		  fprintf(ctxt->output, "\tcd [path]    change directory to path or to root\n");
+		  fprintf(ctxt->output, "\tdir [path]   dumps informations about the node (namespace, attributes, content)\n");
+		  fprintf(ctxt->output, "\tdu [path]    show the structure of the subtree under path or the current node\n");
+		  fprintf(ctxt->output, "\texit         leave shell\n");
+		  fprintf(ctxt->output, "\thelp         display this help\n");
+		  fprintf(ctxt->output, "\tfree         display memory usage\n");
+		  fprintf(ctxt->output, "\tload [name]  load a new document with name\n");
+		  fprintf(ctxt->output, "\tls [path]    list contents of path or the current directory\n");
+#ifdef LIBXML_XPATH_ENABLED
+		  fprintf(ctxt->output, "\txpath expr   evaluate the XPath expression in that context and print the result\n");
+#endif /* LIBXML_XPATH_ENABLED */
+		  fprintf(ctxt->output, "\tpwd          display current working directory\n");
+		  fprintf(ctxt->output, "\tquit         leave shell\n");
+		  fprintf(ctxt->output, "\tsave [name]  save this document to name or the original name\n");
+		  fprintf(ctxt->output, "\tvalidate     check the document for errors\n");
+		  fprintf(ctxt->output, "\twrite [name] write the current node to the filename\n");
         } else if (!strcmp(command, "validate")) {
             xmlShellValidate(ctxt, arg, NULL, NULL);
         } else if (!strcmp(command, "load")) {
@@ -1930,22 +2112,36 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
             xmlShellWrite(ctxt, arg, NULL, NULL);
         } else if (!strcmp(command, "free")) {
             if (arg[0] == 0) {
-                xmlMemShow(stdout, 0);
+                xmlMemShow(ctxt->output, 0);
             } else {
                 int len = 0;
 
                 sscanf(arg, "%d", &len);
-                xmlMemShow(stdout, len);
+                xmlMemShow(ctxt->output, len);
             }
         } else if (!strcmp(command, "pwd")) {
             char dir[500];
 
             if (!xmlShellPwd(ctxt, dir, ctxt->node, NULL))
-                fprintf(stdout, "%s\n", dir);
+                fprintf(ctxt->output, "%s\n", dir);
         } else if (!strcmp(command, "du")) {
             xmlShellDu(ctxt, NULL, ctxt->node, NULL);
         } else if (!strcmp(command, "base")) {
             xmlShellBase(ctxt, NULL, ctxt->node, NULL);
+#ifdef LIBXML_XPATH_ENABLED
+        } else if (!strcmp(command, "xpath")) {
+            if (arg[0] == 0) {
+		xmlGenericError(xmlGenericErrorContext,
+				"xpath: expression required\n");
+	    } else {
+                ctxt->pctxt->node = ctxt->node;
+                list = xmlXPathEval((xmlChar *) arg, ctxt->pctxt);
+		xmlXPathDebugDumpObject(ctxt->output, list, 0);
+		xmlXPathFreeObject(list);
+	    }
+#endif /* LIBXML_XPATH_ENABLED */
+        } else if (!strcmp(command, "setbase")) {
+            xmlShellSetBase(ctxt, arg, ctxt->node, NULL);
         } else if ((!strcmp(command, "ls")) || (!strcmp(command, "dir"))) {
             int dir = (!strcmp(command, "dir"));
 
@@ -2131,7 +2327,7 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
                                      indx < list->nodesetval->nodeNr;
                                      indx++) {
                                     if (i > 0)
-                                        fprintf(stdout, " -------\n");
+                                        fprintf(ctxt->output, " -------\n");
                                     xmlShellCat(ctxt, NULL,
                                                 list->nodesetval->
                                                 nodeTab[indx], NULL);
