@@ -17,9 +17,6 @@
 #else
 #include <libxml/xmlversion.h>
 #endif
-#include <libxml/tree.h>
-#include <libxml/parser.h>
-#include <libxml/encoding.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -30,11 +27,104 @@ extern "C" {
  * I/O structures.
  */
 
+/**
+ * xmlInputMatchCallback:
+ * @filename: the filename or URI
+ *
+ * Callback used in the I/O Input API to detect if the current handler 
+ * can provide input fonctionnalities for this resource.
+ *
+ * Returns 1 if yes and 0 if another Input module should be used
+ */
 typedef int (*xmlInputMatchCallback) (char const *filename);
+/**
+ * xmlInputOpenCallback:
+ * @filename: the filename or URI
+ *
+ * Callback used in the I/O Input API to open the resource
+ *
+ * Returns an Input context or NULL in case or error
+ */
 typedef void * (*xmlInputOpenCallback) (char const *filename);
+/**
+ * xmlInputReadCallback:
+ * @context:  an Input context
+ * @buffer:  the buffer to store data read
+ * @len:  the length of the buffer in bytes
+ *
+ * Callback used in the I/O Input API to read the resource
+ *
+ * Returns the number of bytes read or -1 in case of error
+ */
 typedef int (*xmlInputReadCallback) (void * context, char * buffer, int len);
+/**
+ * xmlInputCloseCallback:
+ * @context:  an Input context
+ *
+ * Callback used in the I/O Input API to close the resource
+ *
+ * Returns 0 or -1 in case of error
+ */
 typedef int (*xmlInputCloseCallback) (void * context);
 
+/*
+ * Those are the functions and datatypes for the library output
+ * I/O structures.
+ */
+
+/**
+ * xmlOutputMatchCallback:
+ * @filename: the filename or URI
+ *
+ * Callback used in the I/O Output API to detect if the current handler 
+ * can provide output fonctionnalities for this resource.
+ *
+ * Returns 1 if yes and 0 if another Output module should be used
+ */
+typedef int (*xmlOutputMatchCallback) (char const *filename);
+/**
+ * xmlOutputOpenCallback:
+ * @filename: the filename or URI
+ *
+ * Callback used in the I/O Output API to open the resource
+ *
+ * Returns an Output context or NULL in case or error
+ */
+typedef void * (*xmlOutputOpenCallback) (char const *filename);
+/**
+ * xmlOutputWriteCallback:
+ * @context:  an Output context
+ * @buffer:  the buffer of data to write
+ * @len:  the length of the buffer in bytes
+ *
+ * Callback used in the I/O Output API to write to the resource
+ *
+ * Returns the number of bytes written or -1 in case of error
+ */
+typedef int (*xmlOutputWriteCallback) (void * context, const char * buffer,
+                                       int len);
+/**
+ * xmlOutputCloseCallback:
+ * @context:  an Output context
+ *
+ * Callback used in the I/O Output API to close the resource
+ *
+ * Returns 0 or -1 in case of error
+ */
+typedef int (*xmlOutputCloseCallback) (void * context);
+
+#ifdef __cplusplus
+}
+#endif
+
+#include <libxml/globals.h>
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/encoding.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 struct _xmlParserInputBuffer {
     void*                  context;
     xmlInputReadCallback   readcallback;
@@ -47,17 +137,6 @@ struct _xmlParserInputBuffer {
 };
 
 
-/*
- * Those are the functions and datatypes for the library output
- * I/O structures.
- */
-
-typedef int (*xmlOutputMatchCallback) (char const *filename);
-typedef void * (*xmlOutputOpenCallback) (char const *filename);
-typedef int (*xmlOutputWriteCallback) (void * context, const char * buffer,
-                                       int len);
-typedef int (*xmlOutputCloseCallback) (void * context);
-
 struct _xmlOutputBuffer {
     void*                   context;
     xmlOutputWriteCallback  writecallback;
@@ -173,6 +252,43 @@ xmlParserInputPtr xmlNoNetExternalEntityLoader(const char *URL,
 					 const char *ID,
 					 xmlParserCtxtPtr ctxt);
 
+xmlChar *xmlNormalizeWindowsPath	(const xmlChar *path);
+
+int	xmlCheckFilename		(const char *path);
+/**
+ * Default 'file://' protocol callbacks 
+ */
+int	xmlFileMatch 			(const char *filename);
+void *	xmlFileOpen 			(const char *filename);
+int	xmlFileRead 			(void * context, 
+					 char * buffer, 
+					 int len);
+int	xmlFileClose 			(void * context);
+
+/**
+ * Default 'http://' protocol callbacks 
+ */
+#ifdef LIBXML_HTTP_ENABLED
+int	xmlIOHTTPMatch 			(const char *filename);
+void *	xmlIOHTTPOpen 			(const char *filename);
+int 	xmlIOHTTPRead			(void * context, 
+					 char * buffer, 
+					 int len);
+int	xmlIOHTTPClose 			(void * context);
+#endif /* LIBXML_HTTP_ENABLED */
+
+/**
+ * Default 'ftp://' protocol callbacks 
+ */
+#ifdef LIBXML_FTP_ENABLED 
+int	xmlIOFTPMatch 			(const char *filename);
+void *	xmlIOFTPOpen 			(const char *filename);
+int 	xmlIOFTPRead			(void * context, 
+					 char * buffer, 
+					 int len);
+int 	xmlIOFTPClose 			(void * context);
+#endif /* LIBXML_FTP_ENABLED */
+
 #ifdef __cplusplus
 }
 #endif
