@@ -268,6 +268,9 @@ py_types = {
     'xmlCatalogPtr': ('O', "catalog", "xmlCatalogPtr", "xmlCatalogPtr"),
     'FILE *': ('O', "File", "FILEPtr", "FILE *"),
     'xmlURIPtr': ('O', "URI", "xmlURIPtr", "xmlURIPtr"),
+    'xmlOutputBufferPtr': ('O', "outputBuffer", "xmlOutputBufferPtr", "xmlOutputBufferPtr"),
+    'xmlParserInputBufferPtr': ('O', "inputBuffer", "xmlParserInputBufferPtr", "xmlParserInputBufferPtr"),
+    'xmlRegexpPtr': ('O', "xmlReg", "xmlRegexpPtr", "xmlRegexpPtr"),
 }
 
 py_return_types = {
@@ -407,6 +410,10 @@ def print_function_wrapper(name, output, export, include):
         include.write("#ifdef LIBXML_XINCLUDE_ENABLED\n");
         export.write("#ifdef LIBXML_XINCLUDE_ENABLED\n");
         output.write("#ifdef LIBXML_XINCLUDE_ENABLED\n");
+    elif file == "xmlregexp":
+        include.write("#ifdef LIBXML_REGEXP_ENABLED\n");
+        export.write("#ifdef LIBXML_REGEXP_ENABLED\n");
+        output.write("#ifdef LIBXML_REGEXP_ENABLED\n");
 
     include.write("PyObject * ")
     include.write("libxml_%s(PyObject *self, PyObject *args);\n" % (name));
@@ -466,6 +473,10 @@ def print_function_wrapper(name, output, export, include):
         include.write("#endif /* LIBXML_XINCLUDE_ENABLED */\n");
         export.write("#endif /* LIBXML_XINCLUDE_ENABLED */\n");
         output.write("#endif /* LIBXML_XINCLUDE_ENABLED */\n");
+    elif file == "xmlregexp":
+        include.write("#endif /* LIBXML_REGEXP_ENABLED */\n");
+        export.write("#endif /* LIBXML_REGEXP_ENABLED */\n");
+        output.write("#endif /* LIBXML_REGEXP_ENABLED */\n");
     return 1
 
 def buildStubs():
@@ -583,6 +594,9 @@ classes_type = {
     "htmlParserCtxt *": ("._o", "parserCtxt(_obj=%s)", "parserCtxt"),
     "xmlCatalogPtr": ("._o", "catalog(_obj=%s)", "catalog"),
     "xmlURIPtr": ("._o", "URI(_obj=%s)", "URI"),
+    "xmlOutputBufferPtr": ("._o", "outputBuffer(_obj=%s)", "outputBuffer"),
+    "xmlParserInputBufferPtr": ("._o", "inputBuffer(_obj=%s)", "inputBuffer"),
+    "xmlRegexpPtr": ("._o", "xmlReg(_obj=%s)", "xmlReg"),
 }
 
 converter_type = {
@@ -600,11 +614,16 @@ classes_ancestor = {
     "xmlEntity" : "xmlNode",
     "xmlElement" : "xmlNode",
     "xmlAttribute" : "xmlNode",
+    "outputBuffer": "ioWriteWrapper",
+    "inputBuffer": "ioReadWrapper",
 }
 classes_destructors = {
     "parserCtxt": "xmlFreeParserCtxt",
     "catalog": "xmlFreeCatalog",
     "URI": "xmlFreeURI",
+#    "outputBuffer": "xmlOutputBufferClose",
+    "inputBuffer": "xmlFreeParserInputBuffer",
+    "xmlReg": "xmlRegFreeRegexp",
 }
 
 functions_noexcept = {
@@ -647,6 +666,16 @@ def nameFixup(name, classe, type, file):
     elif name[0:11] == "xmlXPathSet" and file == "python_accessor":
         func = name[8:]
         func = string.lower(func[0:1]) + func[1:]
+    elif name[0:15] == "xmlOutputBuffer" and file != "python":
+        func = name[15:]
+        func = string.lower(func[0:1]) + func[1:]
+    elif name[0:20] == "xmlParserInputBuffer" and file != "python":
+        func = name[20:]
+        func = string.lower(func[0:1]) + func[1:]
+    elif name[0:9] == "xmlRegexp" and file == "xmlregexp":
+        func = "regexp" + name[9:]
+    elif name[0:6] == "xmlReg" and file == "xmlregexp":
+        func = "regexp" + name[6:]
     elif name[0:11] == "xmlACatalog":
         func = name[11:]
         func = string.lower(func[0:1]) + func[1:]
