@@ -151,35 +151,44 @@ static int xmlOutputCallbackInitialized = 0;
 xmlChar *
 xmlNormalizeWindowsPath(const xmlChar *path)
 {
-    int len, i, j;
+    int len, i = 0, j;
     xmlChar *ret;
 
     if (path == NULL)
 	return(NULL);
-    if (!IS_WINDOWS_PATH(path))
-	return(xmlStrdup(path));
 
     len = xmlStrlen(path);
-    ret = xmlMalloc(len + 10);
-    if (ret == NULL)
-	return(NULL);
+    if (!IS_WINDOWS_PATH(path)) {
+	ret = xmlStrdup(path);
+	if (ret == NULL)
+	    return(NULL);
+	j = 0;
+    } else {
+	ret = xmlMalloc(len + 10);
+	if (ret == NULL)
+	    return(NULL);
+	ret[0] = 'f';
+	ret[1] = 'i';
+	ret[2] = 'l';
+	ret[3] = 'e';
+	ret[4] = ':';
+	ret[5] = '/';
+	ret[6] = '/';
+	ret[7] = '/';
+	j = 8;
+    }
 
-    ret[0] = 'f';
-    ret[1] = 'i';
-    ret[2] = 'l';
-    ret[3] = 'e';
-    ret[4] = ':';
-    ret[5] = '/';
-    ret[6] = '/';
-    ret[7] = '/';
-    for (i = 0,j = 8;i < len;i++,j++) {
+    while (i < len) {
 	/* TODO: UTF8 conversion + URI escaping ??? */
 	if (path[i] == '\\')
 	    ret[j] = '/';
 	else
 	    ret[j] = path[i];
+	i++;
+	j++;
     }
     ret[j] = 0;
+
     return(ret);
 }
 
@@ -252,25 +261,22 @@ xmlCleanupOutputCallbacks(void)
  * returns 1.  if stat fails, returns 0 (if calling
  * stat on the filename fails, it can't be right).
  * if stat succeeds and the file is a directory,
- * sets errno to EISDIR and returns 0.  otherwise
- * returns 1.
+ * returns 2.  otherwise returns 1.
  */
 
-static int
+int
 xmlCheckFilename (const char *path)
 {
 #ifdef HAVE_STAT
-#ifdef S_ISDIR
     struct stat stat_buffer;
 
     if (stat(path, &stat_buffer) == -1)
         return 0;
 
+#ifdef S_ISDIR
     if (S_ISDIR(stat_buffer.st_mode)) {
-        errno = EISDIR;
-        return 0;
+        return 2;
     }
-
 #endif
 #endif
     return 1;
@@ -983,7 +989,7 @@ static void
 xmlFreeHTTPWriteCtxt( xmlIOHTTPWriteCtxtPtr ctxt )
 {
     if ( ctxt->uri != NULL )
-	free( ctxt->uri );
+	xmlFree( ctxt->uri );
 
     if ( ctxt->doc_buff != NULL ) {
 
@@ -998,7 +1004,7 @@ xmlFreeHTTPWriteCtxt( xmlIOHTTPWriteCtxtPtr ctxt )
 	}
     }
 
-    free( ctxt );
+    xmlFree( ctxt );
     return;
 }
 
@@ -2488,6 +2494,10 @@ xmlParserGetDirectory(const char *filename) {
     char *cur;
     char sep = '/';
 
+#ifdef _WIN32_WCE  /* easy way by now ... wince does not have dirs! */
+    return NULL;
+#endif
+
     if (xmlInputCallbackInitialized == 0)
 	xmlRegisterDefaultInputCallbacks();
 
